# -*- coding: utf-8 -*-

# Service registry
# A mapping of method -> type name -> subtype name -> service id
_services = {}


def parse_accept_header(accept):
    """Parse the given Accept header ignoring any parameters and return a list
       of media type tuples.
    """
    media_types = []
    for media_range in accept.split(','):
        media_type = media_range.split(';')[0].strip()
        if '/' in media_type:
            type_, subtype = media_type.split('/')
            media_types.append((type_, subtype))
    return media_types


def lookup_service_id(method, accept):
    """Lookup the service id for the given request method and Accept header.
       Only Accept headers containing exactly one media type are considered for
       negotiation.
    """
    media_types = parse_accept_header(accept)
    if len(media_types) != 1:
        return None
    type_, subtype = media_types[0]
    types = _services.get(method, {})
    subtypes = types.get(type_, {})
    if subtype in subtypes:
        return subtypes[subtype]
    elif '*' in subtypes:
        return subtypes['*']
    if '*' in types:
        return types['*']['*']
    return None


def register_service(method, media_type):
    """Register a service for the given request method and media type and
       return it's service id.
    """
    service_id = u'{}_{}_{}_'.format(method, media_type[0], media_type[1])
    types = _services.setdefault(method, {})
    subtypes = types.setdefault(media_type[0], {})
    subtypes[media_type[1]] = service_id
    return service_id



# -*- coding: utf-8 -*-
from zope.interface import Interface


class IAPIRequest(Interface):
    """Marker for API requests.
    """


class IService(Interface):
    """Marker for REST services.
    """


class ICORSPolicy(Interface):
    """Provides methods for processing simple and preflight CORS requests by
       adding access control headers.
    """
    def process_simple_request():
        """Process a simple request"""

    def process_preflight_request():
        """Process a preflight request"""



# -*- coding: utf-8 -*-
from Products.SiteAccess.VirtualHostMonster import VirtualHostMonster
from ZPublisher import BeforeTraverse
from ZPublisher.pubevents import PubStart
from base64 import b64encode
from plone.app.layout.navigation.interfaces import INavigationRoot
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.rest.service import Service
from plone.rest.testing import PLONE_REST_INTEGRATION_TESTING
from zope.event import notify
from zope.interface import alsoProvides
from zope.publisher.interfaces.browser import IBrowserView

import unittest


class TestTraversal(unittest.TestCase):

    layer = PLONE_REST_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    def traverse(self, path='/plone', accept='application/json', method='GET'):
        request = self.layer['request']
        request.environ['PATH_INFO'] = path
        request.environ['PATH_TRANSLATED'] = path
        request.environ['HTTP_ACCEPT'] = accept
        request.environ['REQUEST_METHOD'] = method
        request._auth = 'Basic %s' % b64encode(
            '%s:%s' % (SITE_OWNER_NAME, SITE_OWNER_PASSWORD))
        notify(PubStart(request))
        return request.traverse(path)

    def test_json_request_on_portal_root_returns_service(self):
        obj = self.traverse()
        self.assertTrue(isinstance(obj, Service), 'Not a service')

    def test_json_request_on_portal_root_with_layout_returns_service(self):
        self.portal.setLayout('summary_view')
        obj = self.traverse()
        self.assertTrue(isinstance(obj, Service), 'Not a service')

    def test_json_request_on_portal_root_with_default_page_returns_service(
            self):
        self.portal.invokeFactory('Document', id='doc1')
        self.portal.setDefaultPage('doc1')
        obj = self.traverse()
        self.assertTrue(isinstance(obj, Service), 'Not a service')

    def test_json_request_on_content_object_returns_service(self):
        self.portal.invokeFactory('Document', id='doc1')
        obj = self.traverse(path='/plone/doc1')
        self.assertTrue(isinstance(obj, Service), 'Not a service')

    def test_html_request_on_portal_root_returns_default_view(self):
        obj = self.traverse(accept='text/html')
        self.assertEquals('listing_view', obj.__name__)

    def test_html_request_on_portal_root_returns_dynamic_view(self):
        self.portal.setLayout('summary_view')
        obj = self.traverse(accept='text/html')
        self.assertEquals('summary_view', obj.__name__)

    def test_html_request_on_portal_root_returns_default_page(self):
        self.portal.invokeFactory('Document', id='doc1')
        self.portal.setDefaultPage('doc1')
        obj = self.traverse(accept='text/html')
        self.assertEquals('document_view', obj.__name__)

    def test_json_request_on_object_with_multihook(self):
        doc1 = self.portal[self.portal.invokeFactory('Document', id='doc1')]

        # Register a function to be called before traversal
        def btr_test(container, request):
            request._btr_test_called = 1
        doc1.btr_test = btr_test
        nc = BeforeTraverse.NameCaller('btr_test')
        BeforeTraverse.registerBeforeTraverse(doc1, nc, 'Document/btr_test')

        obj = self.traverse(path='/plone/doc1')
        self.assertTrue(isinstance(obj, Service), 'Not a service')
        self.assertEquals(1, self.request._btr_test_called)

    def test_json_request_on_existing_view_returns_named_service(self):
        obj = self.traverse('/plone/search')
        self.assertTrue(isinstance(obj, Service), 'Not a service')

        folder = self.portal[self.portal.invokeFactory('Folder', id='folder1')]
        alsoProvides(folder, INavigationRoot)
        obj = self.traverse('/plone/folder1/search')
        self.assertTrue(isinstance(obj, Service), 'Not a service')

    def test_html_request_on_existing_view_returns_view(self):
        obj = self.traverse(path='/plone/search', accept='text/html')
        self.assertFalse(isinstance(obj, Service), 'Got a service')

        folder = self.portal[self.portal.invokeFactory('Folder', id='folder1')]
        alsoProvides(folder, INavigationRoot)
        obj = self.traverse(path='/plone/folder1/search', accept='text/html')
        self.assertFalse(isinstance(obj, Service), 'Got a service')

    def test_virtual_hosting(self):
        app = self.layer['app']
        vhm = VirtualHostMonster()
        vhm.id = 'virtual_hosting'
        vhm.addToContainer(app)
        obj = self.traverse(path='/VirtualHostBase/http/localhost:8080/plone/VirtualHostRoot/')  # noqa
        self.assertTrue(isinstance(obj, Service), 'Not a service')
        del app['virtual_hosting']

    def test_json_request_to_regular_view_returns_view(self):
        obj = self.traverse('/plone/folder_contents')
        self.assertTrue(IBrowserView.providedBy(obj), 'IBrowserView expected')

        self.portal[self.portal.invokeFactory('Folder', id='folder1')]
        obj = self.traverse('/plone/folder1/folder_contents')
        self.assertTrue(IBrowserView.providedBy(obj), 'IBrowserView expected')

    def test_json_request_to_view_namespace_returns_view(self):
        obj = self.traverse('/plone/@@folder_contents')
        self.assertTrue(IBrowserView.providedBy(obj), 'IBrowserView expected')

        self.portal[self.portal.invokeFactory('Folder', id='folder1')]
        obj = self.traverse('/plone/folder1/@@folder_contents')
        self.assertTrue(IBrowserView.providedBy(obj), 'IBrowserView expected')



# -*- coding: utf-8 -*-
from plone.rest.interfaces import ICORSPolicy
from zope.interface import implements

# CORS preflight service registry
# A mapping of method -> service_id
_services = {}


def register_method_for_preflight(method, service_id):
    """Register the given method for preflighting with the given service_id."""
    _services[method] = service_id


def lookup_preflight_service_id(method):
    """Lookup a service id for the given preflighted method."""
    if method in _services:
        return _services[method]


class CORSPolicy(object):
    implements(ICORSPolicy)

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def process_simple_request(self):
        """Process the current request as a simple CORS request by setting the
           appropriate access control headers. Returns True if access control
           headers were set.
        """
        origin = self._allowed_origin()
        if not origin:
            return False

        self._process_origin_and_credentials(origin)

        if self.expose_headers:
            self.request.response.setHeader('Access-Control-Expose-Headers',
                                            ', '.join(self.expose_headers))
        return True

    def process_preflight_request(self):
        """Process the current request as a CORS preflight request by setting
           the appropriate access control headers. Returns True if access
           control headers were set.
        """
        origin = self._allowed_origin()
        if not origin:
            return False

        method = self.request.getHeader('Access-Control-Request-Method', None)
        if self.allow_methods and method not in self.allow_methods:
            return False

        headers = self.request.getHeader('Access-Control-Request-Headers',
                                         None)
        if headers:
            headers = headers.split(',')
            allowed_headers = [h.lower() for h in self.allow_headers]
            for header in headers:
                if header.strip().lower() not in allowed_headers:
                    return False

        self._process_origin_and_credentials(origin)

        if self.max_age:
            self.request.response.setHeader('Access-Control-Max-Age',
                                            self.max_age)

        self.request.response.setHeader('Access-Control-Allow-Methods', method)

        if self.allow_headers:
            self.request.response.setHeader('Access-Control-Allow-Headers',
                                            ', '.join(self.allow_headers))

        self.request.response.setHeader('Content-Length', '0')
        self.request.response.setStatus(200)
        return True

    def _allowed_origin(self):
        origin = self.request.getHeader('Origin', None)
        if not origin:
            return False
        if origin not in self.allow_origin and self.allow_origin != ['*']:
            return False
        return origin

    def _process_origin_and_credentials(self, origin):
        if self.allow_credentials:
            self.request.response.setHeader('Access-Control-Allow-Origin',
                                            origin)
            self.request.response.setHeader('Access-Control-Allow-Credentials',
                                            'true')
            if len(self.allow_origin) > 1 or self.allow_origin == ['*']:
                self.request.response.setHeader('Vary', 'Origin')
        elif self.allow_origin == ['*']:
            self.request.response.setHeader('Access-Control-Allow-Origin', '*')
        else:
            self.request.response.setHeader('Access-Control-Allow-Origin',
                                            origin)
            if len(self.allow_origin) > 1:
                self.request.response.setHeader('Vary', 'Origin')



# -*- coding: utf-8 -*-
from plone.rest import Service

import json


class BaseService(Service):

    def render(self):
        self.request.response.setHeader("Content-Type", "application/json")
        return json.dumps(self.data(), indent=2, sort_keys=True)


class Get(BaseService):

    def data(self):
        return {
            'method': 'GET',
            'id': self.context.id
        }


class Post(BaseService):

    def data(self):
        return {
            'method': 'POST',
            'id': self.context.id
        }


class Put(BaseService):

    def data(self):
        return {
            'method': 'PUT',
            'id': self.context.id
        }


class Delete(BaseService):

    def data(self):
        return {
            'method': 'DELETE',
            'id': self.context.id
        }


class Patch(BaseService):

    def data(self):
        return {
            'method': 'PATCH',
            'id': self.context.id
        }


class Options(BaseService):

    def data(self):
        return {
            'method': 'OPTIONS',
            'id': self.context.id
        }


class NamedGet(BaseService):

    def data(self):
        return {'service': 'named get'}


class NamedPost(BaseService):

    def data(self):
        return {'service': 'named post'}


class NamedPut(BaseService):

    def data(self):
        return {'service': 'named put'}


class NamedDelete(BaseService):

    def data(self):
        return {'service': 'named delete'}


class NamedPatch(BaseService):

    def data(self):
        return {'service': 'named patch'}


class NamedOptions(BaseService):

    def data(self):
        return {'service': 'named options'}



# -*- coding: utf-8 -*-
from BTrees.OOBTree import OOSet
from plone.app.redirector.interfaces import IRedirectionStorage
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.rest.errors import ErrorHandling
from plone.rest.testing import PLONE_REST_FUNCTIONAL_TESTING
from zope.component import queryUtility

import requests
import transaction
import unittest


class TestRedirects(unittest.TestCase):

    layer = PLONE_REST_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer['app']
        self.portal = self.layer['portal']
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

        self.portal.invokeFactory('Folder', id='folder-old')
        self.portal.manage_renameObject('folder-old', 'folder-new')
        transaction.commit()

    def test_get_to_moved_item_causes_301_redirect(self):
        response = requests.get(
            self.portal_url + '/folder-old',
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
            allow_redirects=False,
        )
        self.assertEqual(301, response.status_code)
        self.assertEqual(self.portal_url + '/folder-new',
                         response.headers['Location'])
        self.assertEqual(u'', response.text)

    def test_post_to_moved_item_causes_308_redirect(self):
        response = requests.post(
            self.portal_url + '/folder-old',
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
            allow_redirects=False,
        )
        self.assertEqual(308, response.status_code)
        self.assertEqual(self.portal_url + '/folder-new',
                         response.headers['Location'])
        self.assertEqual(u'', response.text)

    def test_unauthorized_request_to_item_still_redirects_first(self):
        response = requests.get(
            self.portal_url + '/folder-old',
            headers={'Accept': 'application/json'},
            # No auth
            allow_redirects=False,
        )

        # A request to the old URL of an item where the user doesn't have
        # necessary permissions will still result in a redirect
        self.assertEqual(301, response.status_code)
        self.assertEqual(self.portal_url + '/folder-new',
                         response.headers['Location'])
        self.assertEqual(u'', response.text)

        # Following the redirect then leads to an item that will produce a 401
        response = requests.get(
            response.headers['Location'],
            headers={'Accept': 'application/json'},
            # No auth
            allow_redirects=False,
        )
        self.assertEqual(401, response.status_code)

    def test_query_string_gets_preserved(self):
        response = requests.get(
            self.portal_url + '/folder-old?key=value',
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
            allow_redirects=False,
        )
        self.assertEqual(301, response.status_code)
        self.assertEqual(self.portal_url + '/folder-new?key=value',
                         response.headers['Location'])
        self.assertEqual(u'', response.text)

    def test_named_service_on_moved_item_causes_301_redirect(self):
        response = requests.get(
            self.portal_url + '/folder-old/namedservice',
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
            allow_redirects=False,
        )
        self.assertEqual(301, response.status_code)
        self.assertEqual(self.portal_url + '/folder-new/namedservice',
                         response.headers['Location'])
        self.assertEqual(u'', response.text)

    def test_named_service_plus_path_parameter_works(self):
        response = requests.get(
            self.portal_url + '/folder-old/namedservice/param',
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
            allow_redirects=False,
        )
        self.assertEqual(301, response.status_code)
        self.assertEqual(self.portal_url + '/folder-new/namedservice/param',
                         response.headers['Location'])
        self.assertEqual(u'', response.text)

    def test_redirects_for_regular_views_still_work(self):
        response = requests.get(
            self.portal_url + '/folder-old/@@some-view',
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
            allow_redirects=False,
        )
        self.assertEqual(301, response.status_code)
        self.assertEqual(self.portal_url + '/folder-new/@@some-view',
                         response.headers['Location'])
        self.assertEqual(u'', response.text)

    def test_redirects_for_views_plus_params_plus_querystring_works(self):
        response = requests.get(
            self.portal_url + '/folder-old/@@some-view/param?k=v',
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
            allow_redirects=False,
        )
        self.assertEqual(301, response.status_code)
        self.assertEqual(self.portal_url + '/folder-new/@@some-view/param?k=v',
                         response.headers['Location'])
        self.assertEqual(u'', response.text)

    def test_doesnt_cause_redirect_loop_on_bogus_storage_entries(self):
        storage = queryUtility(IRedirectionStorage)
        storage._paths['/plone/same'] = '/plone/same'
        storage._rpaths['/plone/same'] = OOSet(['/plone/same'])
        transaction.commit()

        response = requests.get(
            self.portal_url + '/same/@@view',
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
            allow_redirects=False,
        )
        self.assertEqual(404, response.status_code)

    def test_handles_redirects_that_include_querystring_in_old_path(self):
        storage = queryUtility(IRedirectionStorage)
        storage.add('/plone/some-item?key=value', '/plone/new-item')
        transaction.commit()

        response = requests.get(
            self.portal_url + '/some-item?key=value',
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
            allow_redirects=False,
        )
        self.assertEqual(301, response.status_code)
        self.assertEqual(self.portal_url + '/new-item',
                         response.headers['Location'])
        self.assertEqual(u'', response.text)

    def test_aborts_redirect_checks_early_for_app_root(self):
        error_view = ErrorHandling(self.portal, self.portal.REQUEST)
        self.assertIsNone(
            error_view.find_redirect_if_view_or_service([''], None))

    def test_gracefully_deals_with_missing_request_url(self):
        error_view = ErrorHandling(self.portal, self.portal.REQUEST)
        self.portal.REQUEST['ACTUAL_URL'] = None
        self.assertEquals(False, error_view.attempt_redirect())



# -*- coding: utf-8 -*-
from plone.rest.negotiation import lookup_service_id
from plone.rest.negotiation import parse_accept_header
from plone.rest.negotiation import register_service

import unittest


class TestAcceptHeaderParser(unittest.TestCase):

    def test_parse_application_json_accept_header(self):
        accept = 'application/json'
        expected = [('application', 'json')]
        self.assertEqual(expected, parse_accept_header(accept))

    def test_parse_jquery_json_accept_header(self):
        accept = ('text/javascript, application/javascript, '
                  'application/ecmascript, application/x-ecmascript, '
                  '*/*; q=0.01')
        expected = [('text', 'javascript'),
                    ('application', 'javascript'),
                    ('application', 'ecmascript'),
                    ('application', 'x-ecmascript'),
                    ('*', '*')]
        self.assertEqual(expected, parse_accept_header(accept))

    def test_parse_firefox_accept_header(self):
        accept = ('text/html,application/xhtml+xml,application/xml;q=0.9,'
                  '*/*;q=0.8')
        expected = [('text', 'html'),
                    ('application', 'xhtml+xml'),
                    ('application', 'xml'),
                    ('*', '*')]
        self.assertEqual(expected, parse_accept_header(accept))

    def test_parse_chrome_accept_header(self):
        accept = ('text/html,application/xhtml+xml,application/xml;q=0.9,'
                  'image/webp,*/*;q=0.8')
        expected = [('text', 'html'),
                    ('application', 'xhtml+xml'),
                    ('application', 'xml'),
                    ('image', 'webp'),
                    ('*', '*')]
        self.assertEqual(expected, parse_accept_header(accept))

    def test_parse_all_media_types_accept_header(self):
        self.assertEqual([('*', '*')], parse_accept_header('*/*'))

    def test_parse_invalid_accept_header(self):
        self.assertEqual([], parse_accept_header('invalid'))


class TestServiceRegistry(unittest.TestCase):

    def test_register_media_type(self):
        self.assertEqual(u'GET_application_json_',
                         register_service('GET', ('application', 'json')))
        self.assertEqual(u'GET_application_json_',
                         lookup_service_id('GET', 'application/json'))

    def test_register_wildcard_subtype(self):
        self.assertEqual(u'PATCH_text_*_',
                         register_service('PATCH', ('text', '*')))
        self.assertEqual(u'PATCH_text_*_',
                         lookup_service_id('PATCH', 'text/xml'))

    def test_register_wilcard_type(self):
        self.assertEqual(u'PATCH_*_*_', register_service('PATCH', ('*', '*')))
        self.assertEqual(u'PATCH_*_*_', lookup_service_id('PATCH', 'foo/bar'))

    def test_service_id_for_multiple_media_types_is_none(self):
        register_service('GET', 'application/json')
        self.assertEqual(None, lookup_service_id(
            'GET', 'application/json,application/javascipt'))

    def test_service_id_for_invalid_media_type_is_none(self):
        self.assertEqual(None, lookup_service_id('GET', 'application-json'))

    def test_service_id_for_not_registered_media_type_is_none(self):
        self.assertEqual(None, lookup_service_id('PUT', 'text/html'))

    def test_service_id_for_wildcard_type_is_none(self):
        register_service('GET', 'application/json')
        self.assertEqual(None, lookup_service_id('GET', '*/*'))

    def test_service_id_for_wildcard_subtype_is_none(self):
        register_service('GET', 'text/xml')
        self.assertEqual(None, lookup_service_id('GET', 'text/*'))



# -*- coding: utf-8 -*-
from datetime import datetime
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.textfield.value import RichTextValue
from plone.namedfile.file import NamedBlobImage
from plone.namedfile.file import NamedBlobFile
from plone.rest.testing import PLONE_REST_FUNCTIONAL_TESTING
from z3c.relationfield import RelationValue
from zope.component import getUtility
from zope.intid.interfaces import IIntIds

import unittest
import os
import requests
import transaction


class TestDexterityServiceEndpoints(unittest.TestCase):

    layer = PLONE_REST_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer['app']
        self.request = self.layer['request']
        self.portal = self.layer['portal']
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.portal.invokeFactory('Document', id='doc1')
        self.document = self.portal['doc1']
        transaction.commit()

    def test_dexterity_document_get(self):
        response = requests.get(
            self.document.absolute_url(),
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        )
        self.assertEqual(200, response.status_code)
        self.assertEqual(u'doc1', response.json().get('id'))
        self.assertEqual(u'GET', response.json().get('method'))

    def test_dexterity_document_post(self):
        response = requests.post(
            self.document.absolute_url(),
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        )
        self.assertEqual(200, response.status_code)
        self.assertEqual(u'doc1', response.json().get('id'))
        self.assertEqual(u'POST', response.json().get('method'))

    def test_dexterity_document_put(self):
        response = requests.put(
            self.document.absolute_url(),
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        )
        self.assertEqual(200, response.status_code)
        self.assertEqual(u'doc1', response.json().get('id'))
        self.assertEqual(u'PUT', response.json().get('method'))

    def test_dexterity_document_patch(self):
        response = requests.patch(
            self.document.absolute_url(),
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        )
        self.assertEqual(200, response.status_code)
        self.assertEqual(u'doc1', response.json().get('id'))
        self.assertEqual(u'PATCH', response.json().get('method'))

    def test_dexterity_document_delete(self):
        response = requests.delete(
            self.document.absolute_url(),
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        )
        self.assertEqual(200, response.status_code)
        self.assertEqual(u'doc1', response.json().get('id'))
        self.assertEqual(u'DELETE', response.json().get('method'))

    def test_dexterity_document_options(self):
        response = requests.options(
            self.document.absolute_url(),
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        )
        self.assertEqual(200, response.status_code)
        self.assertEqual(u'doc1', response.json().get('id'))
        self.assertEqual(u'OPTIONS', response.json().get('method'))

    def test_dexterity_folder_get(self):
        self.portal.invokeFactory('Folder', id='folder')
        transaction.commit()

        response = requests.get(
            self.portal.folder.absolute_url(),
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        )

        self.assertEqual(200, response.status_code)
        self.assertEqual(u'folder', response.json().get('id'))
        self.assertEqual(u'GET', response.json().get('method'))

    def test_dexterity_news_item_get(self):
        self.portal.invokeFactory('News Item', id='newsitem')
        self.portal.newsitem.title = 'My News Item'
        self.portal.newsitem.description = u'This is a news item'
        self.portal.newsitem.text = RichTextValue(
            u"Lorem ipsum",
            'text/plain',
            'text/html'
        )
        image_file = os.path.join(os.path.dirname(__file__), u'image.png')
        self.portal.newsitem.image = NamedBlobImage(
            data=open(image_file, 'r').read(),
            contentType='image/png',
            filename=u'image.png'
        )
        self.portal.newsitem.image_caption = u'This is an image caption.'
        import transaction
        transaction.commit()
        response = requests.get(
            self.portal.newsitem.absolute_url(),
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        )

        self.assertEqual(200, response.status_code)
        self.assertEqual(u'newsitem', response.json().get('id'))
        self.assertEqual(u'GET', response.json().get('method'))

    def test_dexterity_event_get(self):
        self.portal.invokeFactory('Event', id='event')
        self.portal.event.title = 'Event'
        self.portal.event.description = u'This is an event'
        self.portal.event.start = datetime(2013, 1, 1, 10, 0)
        self.portal.event.end = datetime(2013, 1, 1, 12, 0)
        import transaction
        transaction.commit()
        response = requests.get(
            self.portal.event.absolute_url(),
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        )

        self.assertEqual(200, response.status_code)
        self.assertEqual(u'event', response.json().get('id'))
        self.assertEqual(u'GET', response.json().get('method'))

    def test_dexterity_link_get(self):
        self.portal.invokeFactory('Link', id='link')
        self.portal.link.title = 'My Link'
        self.portal.link.description = u'This is a link'
        self.portal.remoteUrl = 'http://plone.org'
        import transaction
        transaction.commit()
        response = requests.get(
            self.portal.link.absolute_url(),
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        )

        self.assertEqual(200, response.status_code)
        self.assertEqual(u'link', response.json().get('id'))
        self.assertEqual(u'GET', response.json().get('method'))

    def test_dexterity_file_get(self):
        self.portal.invokeFactory('File', id='file')
        self.portal.file.title = 'My File'
        self.portal.file.description = u'This is a file'
        pdf_file = os.path.join(
            os.path.dirname(__file__), u'file.pdf'
        )
        self.portal.file.file = NamedBlobFile(
            data=open(pdf_file, 'r').read(),
            contentType='application/pdf',
            filename=u'file.pdf'
        )
        intids = getUtility(IIntIds)
        file_id = intids.getId(self.portal.file)
        self.portal.file.file = RelationValue(file_id)
        import transaction
        transaction.commit()

        response = requests.get(
            self.portal.file.absolute_url(),
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        )

        self.assertEqual(200, response.status_code)
        self.assertEqual(u'file', response.json().get('id'))
        self.assertEqual(u'GET', response.json().get('method'))

    def test_dexterity_image_get(self):
        self.portal.invokeFactory('Image', id='image')
        self.portal.image.title = 'My Image'
        self.portal.image.description = u'This is an image'
        image_file = os.path.join(os.path.dirname(__file__), u'image.png')
        self.portal.image.image = NamedBlobImage(
            data=open(image_file, 'r').read(),
            contentType='image/png',
            filename=u'image.png'
        )
        import transaction
        transaction.commit()

        response = requests.get(
            self.portal.image.absolute_url(),
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        )

        self.assertEqual(200, response.status_code)
        self.assertEqual(u'image', response.json().get('id'))
        self.assertEqual(u'GET', response.json().get('method'))

    def test_dexterity_collection_get(self):
        self.portal.invokeFactory('Collection', id='collection')
        self.portal.collection.title = 'My Collection'
        self.portal.collection.description = \
            u'This is a collection with two documents'
        self.portal.collection.query = [{
            'i': 'portal_type',
            'o': 'plone.app.querystring.operation.string.is',
            'v': 'Document',
        }]
        self.portal.invokeFactory(
            'Document',
            id='doc2',
            title='Document 2'
        )
        import transaction
        transaction.commit()
        response = requests.get(
            self.portal.collection.absolute_url(),
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        )

        self.assertEqual(200, response.status_code)
        self.assertEqual(u'collection', response.json().get('id'))
        self.assertEqual(u'GET', response.json().get('method'))



# -*- coding: utf-8 -*-
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.rest.testing import PLONE_REST_FUNCTIONAL_TESTING
from Products.CMFCore.utils import getToolByName

import requests
import transaction
import unittest


CREDS = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
INVALID_CREDS = ('invalid', 'password')
NO_CREDS = ()


class DispatchingTestCase(unittest.TestCase):

    layer = PLONE_REST_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer['app']
        self.request = self.layer['request']
        self.portal = self.layer['portal']
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    def validate(self, expectations, follow_redirects=False):
        failures = []
        for expectation in expectations:
            path, method, creds, expected_status = expectation
            url = self.portal_url + path

            response = requests.request(
                method, url,
                headers={'Accept': 'application/json'},
                auth=creds,
                allow_redirects=follow_redirects,
            )

            if response.status_code != expected_status:
                request_args = (path, method, creds,
                                '(follow_redirects=%s)' % follow_redirects)
                actual_status = response.status_code
                failure = (request_args, expected_status, actual_status)
                failures.append(failure)

        if failures:
            msg = ''
            for (request_args, expected_status, actual_status) in failures:
                msg += ('\n'
                        'Request:  %s\n'
                        'Expected: %s\n'
                        'Got:      %s\n' % (
                            request_args, expected_status, actual_status))

            self.fail('The following assertions failed:\n%s' % msg)


class TestDispatchingSiteRoot(DispatchingTestCase):

    def test_site_root_with_creds(self):
        expectations = [
            ('/', 'GET', CREDS, 200),
            ('/', 'POST', CREDS, 200),
            ('/', 'PUT', CREDS, 200),
            ('/', 'PATCH', CREDS, 200),
            ('/', 'DELETE', CREDS, 200),
            ('/', 'OPTIONS', CREDS, 200),
        ]
        self.validate(expectations)

    def test_site_root_without_creds(self):
        expectations = [
            ('/', 'GET', NO_CREDS, 200),
            ('/', 'POST', NO_CREDS, 401),
            ('/', 'PUT', NO_CREDS, 401),
            ('/', 'PATCH', NO_CREDS, 401),
            ('/', 'DELETE', NO_CREDS, 401),
            ('/', 'OPTIONS', NO_CREDS, 200),
        ]
        self.validate(expectations)

    def test_site_root_invalid_creds(self):
        expectations = [
            ('/', 'GET', INVALID_CREDS, 200),
            ('/', 'POST', INVALID_CREDS, 401),
            ('/', 'PUT', INVALID_CREDS, 401),
            ('/', 'PATCH', INVALID_CREDS, 401),
            ('/', 'DELETE', INVALID_CREDS, 401),
            ('/', 'OPTIONS', INVALID_CREDS, 200),
        ]
        self.validate(expectations)


class TestDispatchingNonExistentResource(DispatchingTestCase):

    def test_not_found_with_creds(self):
        expectations = [
            ('/not-found', 'GET', CREDS, 404),
            ('/not-found', 'POST', CREDS, 404),
            ('/not-found', 'PUT', CREDS, 404),
            ('/not-found', 'PATCH', CREDS, 404),
            ('/not-found', 'DELETE', CREDS, 404),
            ('/not-found', 'OPTIONS', CREDS, 404),
        ]
        self.validate(expectations)

    def test_not_found_without_creds(self):
        expectations = [
            ('/not-found', 'GET', NO_CREDS, 404),
            ('/not-found', 'POST', NO_CREDS, 404),
            ('/not-found', 'PUT', NO_CREDS, 404),
            ('/not-found', 'PATCH', NO_CREDS, 404),
            ('/not-found', 'DELETE', NO_CREDS, 404),
            ('/not-found', 'OPTIONS', NO_CREDS, 404),
        ]
        self.validate(expectations)

    def test_not_found_invalid_creds(self):
        expectations = [
            ('/not-found', 'GET', INVALID_CREDS, 404),
            ('/not-found', 'POST', INVALID_CREDS, 404),
            ('/not-found', 'PUT', INVALID_CREDS, 404),
            ('/not-found', 'PATCH', INVALID_CREDS, 404),
            ('/not-found', 'DELETE', INVALID_CREDS, 404),
            ('/not-found', 'OPTIONS', INVALID_CREDS, 404),
        ]
        self.validate(expectations)


class TestDispatchingDexterity(DispatchingTestCase):

    def setUp(self):
        super(TestDispatchingDexterity, self).setUp()
        self.portal.invokeFactory('Folder', id='private')

        self.portal.invokeFactory('Folder', id='public')
        public_folder = self.portal.public
        wftool = getToolByName(self.portal, "portal_workflow")
        wftool.doActionFor(public_folder, "publish")

        transaction.commit()

    def test_private_dx_folder_with_creds(self):
        expectations = [
            ('/private', 'GET', CREDS, 200),
            ('/private', 'POST', CREDS, 200),
            ('/private', 'PUT', CREDS, 200),
            ('/private', 'PATCH', CREDS, 200),
            ('/private', 'DELETE', CREDS, 200),
            ('/private', 'OPTIONS', CREDS, 200),
        ]
        self.validate(expectations)

    def test_private_dx_folder_without_creds(self):
        expectations = [
            ('/private', 'GET', NO_CREDS, 401),
            ('/private', 'POST', NO_CREDS, 401),
            ('/private', 'PUT', NO_CREDS, 401),
            ('/private', 'PATCH', NO_CREDS, 401),
            ('/private', 'DELETE', NO_CREDS, 401),
            ('/private', 'OPTIONS', NO_CREDS, 401),
        ]
        self.validate(expectations)

    def test_private_dx_folder_invalid_creds(self):
        expectations = [
            ('/private', 'GET', INVALID_CREDS, 401),
            ('/private', 'POST', INVALID_CREDS, 401),
            ('/private', 'PUT', INVALID_CREDS, 401),
            ('/private', 'PATCH', INVALID_CREDS, 401),
            ('/private', 'DELETE', INVALID_CREDS, 401),
            ('/private', 'OPTIONS', INVALID_CREDS, 401),
        ]
        self.validate(expectations)

    def test_public_dx_folder_with_creds(self):
        expectations = [
            ('/public', 'GET', CREDS, 200),
            ('/public', 'POST', CREDS, 200),
            ('/public', 'PUT', CREDS, 200),
            ('/public', 'PATCH', CREDS, 200),
            ('/public', 'DELETE', CREDS, 200),
            ('/public', 'OPTIONS', CREDS, 200),
        ]
        self.validate(expectations)

    def test_public_dx_folder_without_creds(self):
        expectations = [
            ('/public', 'GET', NO_CREDS, 200),
            ('/public', 'POST', NO_CREDS, 401),
            ('/public', 'PUT', NO_CREDS, 401),
            ('/public', 'PATCH', NO_CREDS, 401),
            ('/public', 'DELETE', NO_CREDS, 401),
            ('/public', 'OPTIONS', NO_CREDS, 200),
        ]
        self.validate(expectations)

    def test_public_dx_folder_invalid_creds(self):
        expectations = [
            ('/public', 'GET', INVALID_CREDS, 200),
            ('/public', 'POST', INVALID_CREDS, 401),
            ('/public', 'PUT', INVALID_CREDS, 401),
            ('/public', 'PATCH', INVALID_CREDS, 401),
            ('/public', 'DELETE', INVALID_CREDS, 401),
            ('/public', 'OPTIONS', INVALID_CREDS, 200),
        ]
        self.validate(expectations)


class TestDispatchingRedirects(DispatchingTestCase):

    def setUp(self):
        super(TestDispatchingRedirects, self).setUp()

        self.portal.invokeFactory('Folder', id='private-old')
        self.portal.manage_renameObject('private-old', 'private-new')

        self.portal.invokeFactory('Folder', id='public-old')
        public_folder = self.portal['public-old']
        wftool = getToolByName(self.portal, "portal_workflow")
        wftool.doActionFor(public_folder, "publish")
        self.portal.manage_renameObject('public-old', 'public-new')

        transaction.commit()

    def test_moved_private_dx_folder_with_creds(self):
        expectations = [
            ('/private-old', 'GET', CREDS, 301),
            ('/private-old', 'POST', CREDS, 308),
            ('/private-old', 'PUT', CREDS, 308),
            ('/private-old', 'PATCH', CREDS, 308),
            ('/private-old', 'DELETE', CREDS, 308),
            ('/private-old', 'OPTIONS', CREDS, 308),
        ]
        self.validate(expectations)

        # Same, but with following redirects, asserting on the final status
        expectations = [
            ('/private-old', 'GET', CREDS, 200),
            ('/private-old', 'POST', CREDS, 200),
            ('/private-old', 'PUT', CREDS, 200),
            ('/private-old', 'PATCH', CREDS, 200),
            ('/private-old', 'DELETE', CREDS, 200),
            ('/private-old', 'OPTIONS', CREDS, 200),
        ]
        self.validate(expectations, follow_redirects=True)

    def test_moved_private_dx_folder_without_creds(self):
        expectations = [
            ('/private-old', 'GET', NO_CREDS, 301),
            ('/private-old', 'POST', NO_CREDS, 308),
            ('/private-old', 'PUT', NO_CREDS, 308),
            ('/private-old', 'PATCH', NO_CREDS, 308),
            ('/private-old', 'DELETE', NO_CREDS, 308),
            ('/private-old', 'OPTIONS', NO_CREDS, 308),
        ]
        self.validate(expectations)

        # Same, but with following redirects, asserting on the final status
        expectations = [
            ('/private-old', 'GET', NO_CREDS, 401),
            ('/private-old', 'POST', NO_CREDS, 401),
            ('/private-old', 'PUT', NO_CREDS, 401),
            ('/private-old', 'PATCH', NO_CREDS, 401),
            ('/private-old', 'DELETE', NO_CREDS, 401),
            ('/private-old', 'OPTIONS', NO_CREDS, 401),
        ]
        self.validate(expectations, follow_redirects=True)

    def test_moved_private_dx_folder_invalid_creds(self):
        expectations = [
            ('/private-old', 'GET', INVALID_CREDS, 301),
            ('/private-old', 'POST', INVALID_CREDS, 308),
            ('/private-old', 'PUT', INVALID_CREDS, 308),
            ('/private-old', 'PATCH', INVALID_CREDS, 308),
            ('/private-old', 'DELETE', INVALID_CREDS, 308),
            ('/private-old', 'OPTIONS', INVALID_CREDS, 308),
        ]
        self.validate(expectations)

        # Same, but with following redirects, asserting on the final status
        expectations = [
            ('/private-old', 'GET', INVALID_CREDS, 401),
            ('/private-old', 'POST', INVALID_CREDS, 401),
            ('/private-old', 'PUT', INVALID_CREDS, 401),
            ('/private-old', 'PATCH', INVALID_CREDS, 401),
            ('/private-old', 'DELETE', INVALID_CREDS, 401),
            ('/private-old', 'OPTIONS', INVALID_CREDS, 401),
        ]
        self.validate(expectations, follow_redirects=True)

    def test_moved_public_dx_folder_with_creds(self):
        expectations = [
            ('/public-old', 'GET', CREDS, 301),
            ('/public-old', 'POST', CREDS, 308),
            ('/public-old', 'PUT', CREDS, 308),
            ('/public-old', 'PATCH', CREDS, 308),
            ('/public-old', 'DELETE', CREDS, 308),
            ('/public-old', 'OPTIONS', CREDS, 308),
        ]
        self.validate(expectations)

        # Same, but with following redirects, asserting on the final status
        expectations = [
            ('/public-old', 'GET', CREDS, 200),
            ('/public-old', 'POST', CREDS, 200),
            ('/public-old', 'PUT', CREDS, 200),
            ('/public-old', 'PATCH', CREDS, 200),
            ('/public-old', 'DELETE', CREDS, 200),
            ('/public-old', 'OPTIONS', CREDS, 200),
        ]
        self.validate(expectations, follow_redirects=True)

    def test_moved_public_dx_folder_without_creds(self):
        expectations = [
            ('/public-old', 'GET', NO_CREDS, 301),
            ('/public-old', 'POST', NO_CREDS, 308),
            ('/public-old', 'PUT', NO_CREDS, 308),
            ('/public-old', 'PATCH', NO_CREDS, 308),
            ('/public-old', 'DELETE', NO_CREDS, 308),
            ('/public-old', 'OPTIONS', NO_CREDS, 308),
        ]
        self.validate(expectations)

        # Same, but with following redirects, asserting on the final status
        expectations = [
            ('/public-old', 'GET', NO_CREDS, 200),
            ('/public-old', 'POST', NO_CREDS, 401),
            ('/public-old', 'PUT', NO_CREDS, 401),
            ('/public-old', 'PATCH', NO_CREDS, 401),
            ('/public-old', 'DELETE', NO_CREDS, 401),
            ('/public-old', 'OPTIONS', NO_CREDS, 200),
        ]
        self.validate(expectations, follow_redirects=True)

    def test_moved_public_dx_folder_invalid_creds(self):
        expectations = [
            ('/public-old', 'GET', INVALID_CREDS, 301),
            ('/public-old', 'POST', INVALID_CREDS, 308),
            ('/public-old', 'PUT', INVALID_CREDS, 308),
            ('/public-old', 'PATCH', INVALID_CREDS, 308),
            ('/public-old', 'DELETE', INVALID_CREDS, 308),
            ('/public-old', 'OPTIONS', INVALID_CREDS, 308),
        ]
        self.validate(expectations)

        # Same, but with following redirects, asserting on the final status
        expectations = [
            ('/public-old', 'GET', INVALID_CREDS, 200),
            ('/public-old', 'POST', INVALID_CREDS, 401),
            ('/public-old', 'PUT', INVALID_CREDS, 401),
            ('/public-old', 'PATCH', INVALID_CREDS, 401),
            ('/public-old', 'DELETE', INVALID_CREDS, 401),
            ('/public-old', 'OPTIONS', INVALID_CREDS, 200),
        ]
        self.validate(expectations, follow_redirects=True)



# -*- coding: utf-8 -*-
from plone.rest.interfaces import IAPIRequest


def __before_publishing_traverse__(self, arg1, arg2=None):
    """Pre-traversal hook that stops traversal to prevent the default view
       to be appended. Appending the default view would break REST calls.
    """
    # XXX hack around a bug(?) in BeforeTraverse.MultiHook
    REQUEST = arg2 or arg1

    if IAPIRequest.providedBy(REQUEST):
        return

    return self._old___before_publishing_traverse__(arg1, arg2)


PERMANENT_REDIRECT = {308: 'Permanent Redirect'}


def patch_zpublisher_status_codes(scope, unused_original, unused_replacement):
    """Add '308 Permanent Redirect' to the list of status codes the ZPublisher
    knows about. Otherwise setStatus() will turn it into a 500.

    This is needed for up to and including Plone 5.1.
    """
    status_reasons = getattr(scope, 'status_reasons', {})
    if 308 in status_reasons:
        # Already present in zExceptions >= 3.2 / Zope >= 4.0a1 / Plone 5.2
        return

    # Patch the forward mapping (code -> reason)
    status_reasons.update(PERMANENT_REDIRECT)

    # Update the reverse mapping
    status_codes = getattr(scope, 'status_codes', {})
    key, val = PERMANENT_REDIRECT.items()[0]

    status_codes[''.join(val.split(' ')).lower()] = key
    status_codes[val.lower()] = key
    status_codes[key] = key
    status_codes[str(key)] = key



# -*- coding: utf-8 -*-
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_PASSWORD
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.rest.testing import PLONE_REST_FUNCTIONAL_TESTING

import json
import requests
import transaction
import unittest


class TestErrorHandling(unittest.TestCase):

    layer = PLONE_REST_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer['app']
        self.request = self.layer['request']
        self.portal = self.layer['portal']
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.portal.invokeFactory('Document', id='document1')
        self.document = self.portal.document1
        self.document_url = self.document.absolute_url()
        self.portal.invokeFactory('Folder', id='folder1')
        self.folder = self.portal.folder1
        self.folder_url = self.folder.absolute_url()
        transaction.commit()

    def test_404_not_found(self):
        response = requests.get(
            self.portal_url + '/non-existing-resource',
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        )
        self.assertEqual(response.status_code, 404)
        self.assertEqual(
            response.headers.get('Content-Type'),
            'application/json',
            'When sending a GET request with Accept: application/json ' +
            'the server should respond with sending back application/json.'
        )
        self.assertTrue(json.loads(response.content))
        self.assertEqual(
            'NotFound',
            response.json()['type']
        )
        self.assertEqual(
            'Resource not found: %s' % response.url,
            response.json()['message']
        )

    def test_401_unauthorized(self):
        response = requests.get(
            self.document_url,
            headers={'Accept': 'application/json'}
        )
        self.assertEqual(response.status_code, 401)
        self.assertEqual(
            response.headers.get('Content-Type'),
            'application/json',
            'When sending a GET request with Accept: application/json ' +
            'the server should respond with sending back application/json.'
        )
        self.assertNotIn('Location', response.headers,
                         'A 401 unauthorized should not redirect.')
        self.assertTrue(json.loads(response.content))
        self.assertEqual(
            'Unauthorized',
            response.json()['type']
        )

    def test_500_internal_server_error(self):
        response = requests.get(
            self.portal_url + '/500-internal-server-error',
            headers={'Accept': 'application/json'}
        )

        self.assertEqual(response.status_code, 500)
        self.assertEqual(
            response.headers.get('Content-Type'),
            'application/json',
            'When sending a GET request with Accept: application/json ' +
            'the server should respond with sending back application/json.'
        )
        self.assertTrue(json.loads(response.content))
        self.assertEqual(
            'HTTPError',
            response.json()['type']
        )

        self.assertEqual(
            {u'type': u'HTTPError',
             u'message': u'HTTP Error 500: InternalServerError'},
            response.json())

    def test_500_traceback_only_for_manager_users(self):
        # Normal user
        response = requests.get(
            self.portal_url + '/500-internal-server-error',
            headers={'Accept': 'application/json'},
            auth=(TEST_USER_ID, TEST_USER_PASSWORD)
        )
        self.assertNotIn(u'traceback', response.json())

        # Manager user
        response = requests.get(
            self.portal_url + '/500-internal-server-error',
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        )
        self.assertIn(u'traceback', response.json())

        traceback = response.json()[u'traceback']
        self.assertIsInstance(traceback, list)
        self.assertRegexpMatches(
            traceback[0],
            r'^File "[^"]*", line \d*, in publish')



# -*- coding: utf-8 -*-
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PloneSandboxLayer
from plone.rest.service import Service
from plone.testing import z2

from zope.configuration import xmlconfig


class PloneRestLayer(PloneSandboxLayer):

    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        import plone.rest
        xmlconfig.file(
            'configure.zcml',
            plone.rest,
            context=configurationContext
        )
        xmlconfig.file(
            'testing.zcml',
            plone.rest,
            context=configurationContext
        )


PLONE_REST_FIXTURE = PloneRestLayer()
PLONE_REST_INTEGRATION_TESTING = IntegrationTesting(
    bases=(PLONE_REST_FIXTURE,),
    name="PloneRestLayer:Integration"
)
PLONE_REST_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(PLONE_REST_FIXTURE, z2.ZSERVER_FIXTURE),
    name="PloneRestLayer:Functional"
)


class InternalServerErrorService(Service):

    def __call__(self):
        from urllib2 import HTTPError
        raise HTTPError(
            'http://nohost/plone/500-internal-server-error',
            500,
            'InternalServerError',
            {},
            None
        )



# -*- coding: utf-8 -*-
from AccessControl.class_init import InitializeClass
from AccessControl.security import getSecurityInfo
from AccessControl.security import protectClass
from Products.Five.browser import BrowserView
from plone.rest.cors import CORSPolicy
from plone.rest.cors import register_method_for_preflight
from plone.rest.interfaces import ICORSPolicy
from plone.rest.negotiation import parse_accept_header
from plone.rest.negotiation import register_service
from zope.browserpage.metaconfigure import _handle_for
from zope.component.zcml import handler
from zope.configuration.fields import GlobalInterface
from zope.configuration.fields import GlobalObject
from zope.interface import Interface
from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from zope.schema import Bool
from zope.schema import TextLine
from zope.security.zcml import Permission


class IService(Interface):
    """
    """

    method = TextLine(
        title=u"The name of the view that should be the default. " +
              u"[get|post|put|delete]",
        description=u"""
        This name refers to view that should be the view used by
        default (if no view name is supplied explicitly).""",
        )

    accept = TextLine(
        title=u"Acceptable media types",
        description=u"""Specifies the media type used for content negotiation.
        The service is limited to the given media type and only called if the
        request contains an "Accept" header with the given media type. Multiple
        media types can be given by separating them with a comma.""",
        default=u"application/json")

    for_ = GlobalObject(
        title=u"The interface this view is the default for.",
        description=u"""Specifies the interface for which the view is
        registered. All objects implementing this interface can make use of
        this view. If this attribute is not specified, the view is available
        for all objects.""",
        )

    factory = GlobalObject(
        title=u"The factory for this service",
        description=u"The factory is usually subclass of the Service class.")

    name = TextLine(
        title=u"The name of the service.",
        description=u"""When no name is defined, the service is available at
        the object's absolute URL. When defining a name, the service is
        available at the object's absolute URL appended with a slash and the
        service name.""",
        required=False,
        default=u'')

    layer = GlobalInterface(
        title=u"The browser layer for which this service is registered.",
        description=u"""Useful for overriding existing services or for making
                        services available only if a specific add-on has been
                        installed.""",
        required=False,
        default=IDefaultBrowserLayer,
        )

    permission = Permission(
        title=u"Permission",
        description=u"The permission needed to access the service.",
        required=True,
        )


def serviceDirective(
        _context,
        method,
        accept,
        factory,
        for_,
        permission,
        layer=IDefaultBrowserLayer,
        name=u'',
        ):

    _handle_for(_context, for_)

    media_types = parse_accept_header(accept)
    for media_type in media_types:
        service_id = register_service(method.upper(), media_type)
        view_name = service_id + name

        # We need a service for CORS preflight processing but we don't get the
        # desired Accept header in the preflight request. Thus we just register
        # the current service_id for the given method.
        register_method_for_preflight(method.upper(), service_id)

        # Create a new class. We'll execute some security declarations on it
        # and don't want to modify the original class.
        cdict = getSecurityInfo(factory)
        cdict['__name__'] = view_name
        cdict['method'] = method.upper()
        new_class = type(factory.__name__, (factory, BrowserView), cdict)

        _context.action(
            discriminator=('plone.rest:service', method, media_type, for_,
                           name, layer),
            callable=handler,
            args=('registerAdapter', new_class, (for_, layer), Interface,
                  view_name, _context.info),
        )

        _context.action(
            discriminator=('plone.rest:protectClass', new_class),
            callable=protectClass,
            args=(new_class, permission)
        )
        _context.action(
            discriminator=('plone.rest:InitializeClass', new_class),
            callable=InitializeClass,
            args=(new_class,)
            )


class ICORSPolicyDirective(Interface):
    """Directive for defining CORS policies"""

    for_ = GlobalObject(
        title=u"The interface this CORS policy is for.",
        description=u"""Specifies the interface for which the CORS policy is
        registered. If this attribute is not specified, the CORS policy applies
        to all objects.""",
        required=False,
        )

    layer = GlobalInterface(
        title=u"The browser layer for which this CORS policy is registered.",
        description=u"""Useful for overriding existing policies or for making
                        them available only if a specific add-on has been
                        installed.""",
        required=False,
        default=IDefaultBrowserLayer,
        )

    allow_origin = TextLine(
        title=u"Origins",
        description=u"""Origins that are allowed access to the resource. Either
        a comma separated list of origins, e.g. "http://example.net,
        http://mydomain.com" or "*".""",
        )

    allow_methods = TextLine(
        title=u"Methods",
        description=u"""A comma separated list of HTTP method names that are
        allowed by this CORS policy, e.g. "DELETE,GET,OPTIONS,PATCH,POST,PUT".
        """,
        required=False,
        )

    allow_headers = TextLine(
        title=u"Headers",
        description=u"""A comma separated list of request headers allowed to be
        sent by the client, e.g. "X-My-Header".""",
        required=False,
        )

    expose_headers = TextLine(
        title=u"Exposed Headers",
        description=u"""A comma separated list of response headers clients can
        access, e.g. "Content-Length,X-My-Header".""",
        required=False,
        )

    allow_credentials = Bool(
        title=u"Support Credentials",
        description=u"""Indicates whether the resource supports user
        credentials in the request.""",
        default=False,
        )

    max_age = TextLine(
        title=u"Max Age",
        description=u"""Indicates how long the results of a preflight request
        can be cached.""",
        required=False,
        )


def cors_policy_directive(
        _context,
        allow_origin,
        allow_credentials,
        allow_methods=None,
        expose_headers=None,
        allow_headers=None,
        max_age=None,
        for_=Interface,
        layer=IDefaultBrowserLayer,):

    _handle_for(_context, for_)

    # Create a new policy class and store the CORS policy configuration in
    # class attributes.
    cdict = {}
    cdict['allow_origin'] = [o.strip() for o in allow_origin.split(',')]
    if allow_methods is not None:
        cdict['allow_methods'] = [m.strip() for m in allow_methods.split(',')]
    else:
        cdict['allow_methods'] = None
    cdict['allow_credentials'] = allow_credentials
    if expose_headers:
        cdict['expose_headers'] = [
            h.strip() for h in expose_headers.split(',')]
    else:
        cdict['expose_headers'] = []
    if allow_headers:
        cdict['allow_headers'] = [h.strip() for h in allow_headers.split(',')]
    else:
        cdict['allow_headers'] = []
    cdict['max_age'] = max_age
    new_class = type(CORSPolicy.__name__, (CORSPolicy,), cdict)

    _context.action(
        discriminator=('plone.rest:CORSPolicy', for_, layer),
        callable=handler,
        args=('registerAdapter', new_class, (for_, layer),
              ICORSPolicy, u'', _context.info),
        )



# -*- coding: utf-8 -*-
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.rest.testing import PLONE_REST_FUNCTIONAL_TESTING

import unittest
import requests
import transaction


class TestNamedServiceEndpoints(unittest.TestCase):

    layer = PLONE_REST_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer['app']
        self.request = self.layer['request']
        self.portal = self.layer['portal']
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.portal.invokeFactory('Document', id='doc1')
        self.document = self.portal['doc1']
        transaction.commit()

    def test_dexterity_named_get(self):
        response = requests.get(
            self.document.absolute_url() + '/namedservice',
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            {u'service': u'named get'},
            response.json()
        )

    def test_dexterity_named_post(self):
        response = requests.post(
            self.document.absolute_url() + '/namedservice',
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            {u'service': u'named post'},
            response.json()
        )

    def test_dexterity_named_put(self):
        response = requests.put(
            self.document.absolute_url() + '/namedservice',
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            {u'service': u'named put'},
            response.json()
        )

    def test_dexterity_named_patch(self):
        response = requests.patch(
            self.document.absolute_url() + '/namedservice',
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            {u'service': u'named patch'},
            response.json()
        )

    def test_dexterity_named_delete(self):
        response = requests.delete(
            self.document.absolute_url() + '/namedservice',
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            {u'service': u'named delete'},
            response.json()
        )

    def test_dexterity_named_options(self):
        response = requests.options(
            self.document.absolute_url() + '/namedservice',
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            {u'service': u'named options'},
            response.json()
        )



from AccessControl import getSecurityManager
from plone.app.redirector.interfaces import IRedirectionStorage
from plone.memoize.instance import memoize
from plone.rest.interfaces import IAPIRequest
from Products.CMFCore.permissions import ManagePortal
from Products.Five.browser import BrowserView
from six.moves import urllib
from six.moves.urllib.parse import quote
from six.moves.urllib.parse import unquote
from zExceptions import NotFound
from zope.component import adapter
from zope.component import queryUtility
from zope.component.hooks import getSite

import json
import sys
import traceback


@adapter(Exception, IAPIRequest)
class ErrorHandling(BrowserView):
    """This view is responsible for serializing unhandled exceptions, as well
    as handling 404 Not Found errors and redirects.
    """

    def __call__(self):
        exception = self.context
        data = self.render_exception(exception)
        result = json.dumps(data, indent=2, sort_keys=True)

        # Write and lock the response in order to avoid later changes
        # especially for Unauthorized exceptions.
        response = self.request.response
        response.setHeader('Content-Type', 'application/json')
        response.setStatus(type(exception), lock=1)
        response.setBody(result, lock=1)

        # Avoid redirect to login page on Unauthorized by adding
        # a fake challenged flag, which makes the PAS believe it
        # already did challenge and redirect.
        response._has_challenged = True
        return

    def render_exception(self, exception):
        result = {u'type': type(exception).__name__.decode('utf-8'),
                  u'message': str(exception).decode('utf-8')}

        if isinstance(exception, NotFound):
            # First check if a redirect from p.a.redirector exists
            redirect_performed = self.attempt_redirect()
            if redirect_performed:
                self.request.response.setBody('', lock=1)
                return

            # NotFound exceptions need special handling because their
            # exception message gets turned into HTML by ZPublisher
            url = self.request.getURL()
            result[u'message'] = u'Resource not found: %s' % url

        if getSecurityManager().checkPermission(ManagePortal, getSite()):
            result[u'traceback'] = self.render_traceback(exception)

        return result

    def render_traceback(self, exception):
        _, exc_obj, exc_traceback = sys.exc_info()
        if exception is not exc_obj:
            return (u'ERROR: Another exception happened before we could '
                    u'render the traceback.')

        raw = '\n'.join(traceback.format_tb(exc_traceback))
        return raw.strip().split('\n')

    def find_redirect_if_view_or_service(self, old_path_elements, storage):
        """Find redirect for URLs like:
        - http://example.com/object/namedservice/param
        - http://example.com/object/@@view/param
        - http://example.com/object/template

        This combines the functionality of the find_redirect_if_view() and
        find_redirect_if_template() methods of the original FourOhFourView into
        one, and also makes it support named services.

        For this to also work for named services we use a different strategy
        here: Based on old_path_elements, try to find the longest stored
        redirect (if any), and consider the remaining path parts the remainder
        (view, template, named services plus possible params) that will need
        to be appended to the new object path.
        """
        if len(old_path_elements) <= 1:
            return None

        # Parts to the left of the split point are considered a potential
        # object path, while the right part is the remainder. Starting from
        # the right (longest potential obj path), we keep moving the split
        # point to the left and look for shorter matches.
        #
        # Once we reach the point where the obj path is separated from the
        # remainder, we should get a match if there's a stored redirect.
        #
        # ['', 'Plone', 'folder', 'item', '@@view', 'param']
        #                                ^
        splitpoint = len(old_path_elements)

        while splitpoint > 1:
            possible_obj_path = '/'.join(old_path_elements[:splitpoint])
            remainder = old_path_elements[splitpoint:]
            new_path = storage.get(possible_obj_path)

            if new_path:
                if new_path == possible_obj_path:
                    # New URL would match originally requested URL.
                    # Lets not cause a redirect loop.
                    return None
                return new_path + '/' + '/'.join(remainder)

            splitpoint -= 1

        return None

    def attempt_redirect(self):
        """Check if a redirect is needed, and perform it if necessary.

        Returns True if a redirect has been performed, False otherwise.

        This method is based on FourOhFourView.attempt_redirect() from
        p.a.redirector. It's copied here because we want to answer redirects
        to non-GET methods with status 308, but since this method locks the
        response status, we wouldn't be able to change it afterwards.
        """
        url = self._url()
        if not url:
            return False

        try:
            old_path_elements = self.request.physicalPathFromURL(url)
        except ValueError:  # pragma: no cover
            return False    # pragma: no cover

        storage = queryUtility(IRedirectionStorage)
        if storage is None:
            return False

        old_path = '/'.join(old_path_elements)

        # First lets try with query string in cases or content migration

        new_path = None

        query_string = self.request.QUERY_STRING
        if query_string:
            new_path = storage.get("%s?%s" % (old_path, query_string))
            # if we matched on the query_string we don't want to include it
            # in redirect
            if new_path:
                query_string = ''

        if not new_path:
            new_path = storage.get(old_path)

        # Attempt our own strategy at finding redirects for named REST
        # services, views or templates.
        if not new_path:
            new_path = self.find_redirect_if_view_or_service(
                old_path_elements, storage)

        if not new_path:
            return False

        url = urllib.parse.urlsplit(new_path)
        if url.netloc:
            # External URL
            # avoid double quoting
            url_path = unquote(url.path)
            url_path = quote(url_path)
            url = urllib.parse.SplitResult(
                *(url[:2] + (url_path, ) + url[3:])).geturl()
        else:
            url = self.request.physicalPathToURL(new_path)

        # some analytics programs might use this info to track
        if query_string:
            url += "?" + query_string

        # Answer GET requests with 301. Every other method will be answered
        # with 308 Permanent Redirect, which instructs the client to NOT
        # switch the method (if the original request was a POST, it should
        # re-POST to the new URL from the Location header).
        if self.request.method.upper() == 'GET':
            status = 301
        else:
            status = 308

        self.request.response.redirect(url, status=status, lock=1)
        return True

    @memoize
    def _url(self):
        """Get the current, canonical URL
        """
        return self.request.get('ACTUAL_URL',
            self.request.get('VIRTUAL_URL',  # noqa
                   self.request.get('URL',   # noqa
                     None)))                 # noqa



# -*- coding: utf-8 -*-
from plone.rest.interfaces import ICORSPolicy
from plone.rest.interfaces import IService
from zope.component import queryMultiAdapter
from zope.interface import implements


class Service(object):
    implements(IService)

    def __call__(self):
        policy = queryMultiAdapter((self.context, self.request), ICORSPolicy)
        if policy is not None:
            if self.request._rest_cors_preflight:
                policy.process_preflight_request()
                return
            else:
                policy.process_simple_request()
        else:
            if self.request._rest_cors_preflight:
                return

        return self.render()

    def render(self):
        raise NotImplementedError

    def __getattribute__(self, name):
        # Preflight requests need to be publicly accessible since they don't
        # include credentials
        if name == '__roles__' and self.request._rest_cors_preflight:
            return ['Anonymous']
        return super(Service, self).__getattribute__(name)



# -*- coding: utf-8 -*-
from Products.SiteAccess.VirtualHostMonster import VirtualHostMonster
from ZPublisher.BaseRequest import DefaultPublishTraverse
from plone.rest.interfaces import IAPIRequest
from plone.rest.interfaces import IService
from zope.component import adapts
from zope.component import queryMultiAdapter
from zope.interface import implements
from zope.publisher.interfaces.browser import IBrowserPublisher
from Products.CMFCore.interfaces import IContentish
from Products.CMFCore.interfaces import ISiteRoot


class RESTTraverse(DefaultPublishTraverse):
    adapts(ISiteRoot, IAPIRequest)

    def publishTraverse(self, request, name):
        try:
            obj = super(RESTTraverse, self).publishTraverse(request, name)
            if (not IContentish.providedBy(obj)
                    and not IService.providedBy(obj)):
                if isinstance(obj, VirtualHostMonster):
                    return obj
                else:
                    raise KeyError
        except KeyError:
            # No object, maybe a named rest service
            service = queryMultiAdapter((self.context, request),
                                        name=request._rest_service_id + name)
            if service is None:
                # No service, fallback to regular view
                view = queryMultiAdapter((self.context, request), name=name)
                if view is not None:
                    return view
                raise
            return service

        if name.startswith(request._rest_service_id):
            return obj

        # Do not handle view namespace
        if '@@' in request['PATH_INFO'] or '++view++' in request['PATH_INFO']:
            return obj

        # Wrap object to ensure we handle further traversal
        return RESTWrapper(obj, request)

    def browserDefault(self, request):
        # Called when we have reached the end of the path
        # In our case this means an unamed service
        return self.context, (request._rest_service_id,)


class RESTWrapper(object):
    """A wrapper for objects traversed during a REST request.
    """
    implements(IBrowserPublisher)

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self._bpth_called = False

    def __getattr__(self, name):
        # Delegate attribute access to the wrapped object
        # Needed for security checks in ZPublisher traversal
        return getattr(self.context, name)

    def __getitem__(self, name):
        # Delegate key access to the wrapped object
        return self.context[name]

    # MultiHook requries this to be a class attribute
    def __before_publishing_traverse__(self, arg1, arg2=None):
        bpth = getattr(self.context, '__before_publishing_traverse__', False)
        if bpth:
            if not self._bpth_called:
                self._bpth_called = True
                bpth(arg1, arg2)

    def publishTraverse(self, request, name):
        # Try to get an object using default traversal
        adapter = DefaultPublishTraverse(self.context, request)
        try:
            obj = adapter.publishTraverse(request, name)
            if (not IContentish.providedBy(obj)
                    and not IService.providedBy(obj)):
                raise KeyError

        # If there's no object with the given name, we get a KeyError.
        # In a non-folderish context a key lookup results in an AttributeError.
        except (KeyError, AttributeError):
            # No object, maybe a named rest service
            service = queryMultiAdapter((self.context, request),
                                        name=request._rest_service_id + name)
            if service is None:
                # No service, fallback to regular view
                view = queryMultiAdapter((self.context, request), name=name)
                if view is not None:
                    return view
                raise
            return service
        else:
            # Wrap object to ensure we handle further traversal
            return RESTWrapper(obj, request)

    def browserDefault(self, request):
        # Called when we have reached the end of the path
        # In our case this means an unamed service
        return self.context, (request._rest_service_id,)



# -*- coding: utf-8 -*-
from plone.rest.testing import PLONE_REST_FUNCTIONAL_TESTING
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD

import unittest
import requests


class TestSiteRootServiceEndpoints(unittest.TestCase):

    layer = PLONE_REST_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer['app']
        self.request = self.layer['request']
        self.portal = self.layer['portal']
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    def test_siteroot_get(self):
        response = requests.get(
            self.portal.absolute_url(),
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        )
        self.assertEqual(
            response.status_code,
            200,
            'GET /Plone should return status code 200, not {}'.format(
                response.status_code
            )
        )
        self.assertEqual(u'plone', response.json().get('id'))
        self.assertEqual(u'GET', response.json().get('method'))

    def test_siteroot_post(self):
        response = requests.post(
            self.portal.absolute_url(),
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        )
        self.assertEqual(
            response.status_code,
            200,
            'POST /Plone should return status code 201, not {}'.format(
                response.status_code
            )
        )
        self.assertEqual(u'plone', response.json().get('id'))
        self.assertEqual(u'POST', response.json().get('method'))

    def test_siteroot_delete(self):
        response = requests.delete(
            self.portal.absolute_url(),
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(u'plone', response.json().get('id'))
        self.assertEqual(u'DELETE', response.json().get('method'))

    def test_siteroot_put(self):
        response = requests.put(
            self.portal.absolute_url(),
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(u'plone', response.json().get('id'))
        self.assertEqual(u'PUT', response.json().get('method'))

    def test_siteroot_patch(self):
        response = requests.patch(
            self.portal.absolute_url(),
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(u'plone', response.json().get('id'))
        self.assertEqual(u'PATCH', response.json().get('method'))

    def test_siteroot_options(self):
        response = requests.options(
            self.portal.absolute_url(),
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(u'plone', response.json().get('id'))
        self.assertEqual(u'OPTIONS', response.json().get('method'))



# -*- coding: utf-8 -*-
from Products.CMFCore.utils import getToolByName
from ZPublisher.pubevents import PubStart
from base64 import b64encode
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.rest.service import Service
from plone.rest.testing import PLONE_REST_INTEGRATION_TESTING
from zExceptions import Unauthorized
from zope.event import notify

import unittest


class TestPermissions(unittest.TestCase):

    layer = PLONE_REST_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        setRoles(self.portal, TEST_USER_ID, ['Member'])
        self.portal.acl_users.userFolderAddUser(
            'admin',
            'secret',
            ['Manager'],
            []
        )
        login(self.portal, SITE_OWNER_NAME)
        self.portal.invokeFactory('Document', id='doc1')
        wftool = getToolByName(self.portal, 'portal_workflow')
        wftool.doActionFor(self.portal.doc1, 'publish')

    def traverse(self, path='/plone', accept='application/json', method='GET'):
        request = self.layer['request']
        request.environ['PATH_INFO'] = path
        request.environ['PATH_TRANSLATED'] = path
        request.environ['HTTP_ACCEPT'] = accept
        request.environ['REQUEST_METHOD'] = method
        request._auth = 'Basic %s' % b64encode(
            '%s:%s' % (TEST_USER_NAME, TEST_USER_PASSWORD))
        notify(PubStart(request))
        return request.traverse(path)

    def test_traverse_with_insufficient_permission_raises_unauthorized(self):
        setRoles(self.portal, TEST_USER_ID, ['Member'])
        with self.assertRaises(Unauthorized):
            self.traverse('/plone/doc1', method='PUT')

    def test_traverse_with_sufficient_permission_returns_service(self):
        setRoles(self.portal, TEST_USER_ID, ['Editor'])
        obj = self.traverse('/plone/doc1', method='PUT')
        self.assertTrue(isinstance(obj, Service), 'Not a service')



# -*- coding: utf-8 -*-
from ZPublisher.pubevents import PubStart
from plone.app.testing import popGlobalRegistry
from plone.app.testing import pushGlobalRegistry
from plone.rest.cors import CORSPolicy
from plone.rest.interfaces import ICORSPolicy
from plone.rest.testing import PLONE_REST_INTEGRATION_TESTING
from zExceptions import Unauthorized
from zope.component import provideAdapter
from zope.event import notify
from zope.interface import Interface
from zope.publisher.interfaces.browser import IDefaultBrowserLayer

import unittest


class TestCORSPolicy(unittest.TestCase):

    layer = PLONE_REST_INTEGRATION_TESTING

    def setUp(self):
        self.request = self.layer['request']
        self.response = self.request.response

    def get_policy(self, origin=None, method=None, headers=None):
        request = self.request
        if origin:
            request.environ['HTTP_ORIGIN'] = origin
        if method:
            request.environ['HTTP_ACCESS_CONTROL_REQUEST_METHOD'] = method
        if headers:
            request.environ['HTTP_ACCESS_CONTROL_REQUEST_HEADERS'] = headers
        policy = CORSPolicy(None, request)
        policy.allow_origin = ['*']
        policy.allow_credentials = False
        policy.allow_methods = [
            'DELETE', 'GET', 'OPTIONS', 'PATCH', 'POST', 'PUT']
        policy.allow_headers = []
        policy.expose_headers = []
        policy.max_age = None
        return policy

    def test_simple_cors_without_origin_doesnt_add_ac_headers(self):
        policy = self.get_policy()
        self.assertFalse(policy.process_simple_request())
        self.assertEqual(
            None,
            self.response.getHeader('Access-Control-Allow-Origin'))

    def test_simple_cors_with_wrong_origin_doesnt_add_ac_headers(self):
        policy = self.get_policy(origin="http://wrong.net")
        policy.allow_origin = ['http://example.net']
        self.assertFalse(policy.process_simple_request())
        self.assertEqual(
            None,
            self.response.getHeader('Access-Control-Allow-Origin'))

    def test_simple_cors_adds_wildcard_allow_origin(self):
        policy = self.get_policy(origin="http://example.net")
        self.assertTrue(policy.process_simple_request())
        self.assertEqual(
            '*', self.response.getHeader('Access-Control-Allow-Origin'))

    def test_simple_cors_adds_matching_allow_origin(self):
        policy = self.get_policy(origin="http://example.net")
        policy.allow_origin = ['http://example.net']
        self.assertTrue(policy.process_simple_request())
        self.assertEqual(
            'http://example.net',
            self.response.getHeader('Access-Control-Allow-Origin'))

    def test_simple_cors_adds_allow_credentials(self):
        policy = self.get_policy(origin="http://example.net")
        policy.allow_origin = ['http://example.net']
        policy.allow_credentials = True
        self.assertTrue(policy.process_simple_request())
        self.assertEqual(
            'true',
            self.response.getHeader('Access-Control-Allow-Credentials'))

    def test_simple_cors_adds_origin_when_supporting_credentials(self):
        policy = self.get_policy(origin="http://example.net")
        policy.allow_credentials = True
        self.assertTrue(policy.process_simple_request())
        self.assertEqual(
            'http://example.net',
            self.response.getHeader('Access-Control-Allow-Origin'))
        self.assertEqual(
            'true',
            self.response.getHeader('Access-Control-Allow-Credentials'))

    def test_simple_cors_adds_vary_when_allowing_multiple_origins(self):
        policy = self.get_policy(origin="http://example.net")
        policy.allow_origin = ['http://some.host', 'http://example.net']
        self.assertTrue(policy.process_simple_request())
        self.assertEqual('Origin', self.response.getHeader('Vary'))

    def test_simple_cors_adds_exposed_headers(self):
        policy = self.get_policy(origin="http://example.net")
        policy.expose_headers = ['Content-Length', 'X-My-Header']
        self.assertTrue(policy.process_simple_request())
        self.assertEqual(
            'Content-Length, X-My-Header',
            self.response.getHeader('Access-Control-Expose-Headers'))

    def test_preflight_cors_without_origin_doesnt_add_ac_headers(self):
        policy = self.get_policy()
        self.assertFalse(policy.process_preflight_request())
        self.assertEqual(
            None,
            self.response.getHeader('Access-Control-Allow-Origin'))

    def test_preflight_cors_with_wrong_origin_doesnt_add_ac_headers(self):
        policy = self.get_policy(origin="http://wrong.net")
        policy.allow_origin = ['http://example.net']
        self.assertFalse(policy.process_preflight_request())
        self.assertEqual(
            None,
            self.response.getHeader('Access-Control-Allow-Origin'))

    def test_preflight_cors_with_wrong_method_doesnt_add_ac_headers(self):
        policy = self.get_policy(origin="http://example.net", method="LOCK")
        self.assertFalse(policy.process_preflight_request())
        self.assertEqual(
            None,
            self.response.getHeader('Access-Control-Allow-Origin'))

    def test_preflight_cors_with_wrong_header_doesnt_add_ac_headers(self):
        policy = self.get_policy(
            origin="http://example.net", headers='X-Secret')
        self.assertFalse(policy.process_preflight_request())
        self.assertEqual(
            None,
            self.response.getHeader('Access-Control-Allow-Origin'))

    def test_preflight_cors_adds_wildcard_allow_origin(self):
        policy = self.get_policy(origin="http://example.net", method='GET')
        self.assertTrue(policy.process_preflight_request())
        self.assertEqual(
            '*', self.response.getHeader('Access-Control-Allow-Origin'))

    def test_preflight_cors_adds_matching_allow_origin(self):
        policy = self.get_policy(origin="http://example.net", method='GET')
        policy.allow_origin = ['http://example.net']
        self.assertTrue(policy.process_preflight_request())
        self.assertEqual(
            'http://example.net',
            self.response.getHeader('Access-Control-Allow-Origin'))

    def test_preflight_cors_adds_allow_credentials(self):
        policy = self.get_policy(origin="http://example.net", method='GET')
        policy.allow_origin = ['http://example.net']
        policy.allow_credentials = True
        self.assertTrue(policy.process_preflight_request())
        self.assertEqual(
            'true',
            self.response.getHeader('Access-Control-Allow-Credentials'))

    def test_preflight_cors_adds_origin_when_supporting_credentials(self):
        policy = self.get_policy(origin="http://example.net", method='GET')
        policy.allow_credentials = True
        self.assertTrue(policy.process_preflight_request())
        self.assertEqual(
            'http://example.net',
            self.response.getHeader('Access-Control-Allow-Origin'))
        self.assertEqual(
            'true',
            self.response.getHeader('Access-Control-Allow-Credentials'))

    def test_preflight_cors_adds_vary_when_allowing_multiple_origins(self):
        policy = self.get_policy(origin="http://example.net", method='GET')
        policy.allow_origin = ['http://some.host', 'http://example.net']
        self.assertTrue(policy.process_preflight_request())
        self.assertEqual('Origin', self.response.getHeader('Vary'))

    def test_preflight_cors_adds_max_age(self):
        policy = self.get_policy(origin="http://example.net", method='GET')
        policy.max_age = '3600'
        self.assertTrue(policy.process_preflight_request())
        self.assertEqual(
            '3600', self.response.getHeader('Access-Control-Max-Age'))

    def test_preflight_cors_adds_allow_methods(self):
        policy = self.get_policy(origin="http://example.net", method='GET')
        self.assertTrue(policy.process_preflight_request())
        self.assertEqual(
            'GET', self.response.getHeader('Access-Control-Allow-Methods'))

    def test_preflight_cors_adds_allow_methods_if_unrestriced(self):
        policy = self.get_policy(origin="http://example.net", method='GET')
        policy.allow_methods = None
        self.assertTrue(policy.process_preflight_request())
        self.assertEqual(
            'GET', self.response.getHeader('Access-Control-Allow-Methods'))

    def test_preflight_cors_adds_allow_headers(self):
        policy = self.get_policy(origin="http://example.net", method='GET')
        policy.allow_headers = ['X-Allowed']
        self.assertTrue(policy.process_preflight_request())
        self.assertEqual(
            'X-Allowed',
            self.response.getHeader('Access-Control-Allow-Headers'))

    def test_preflight_cors_sets_content_length_zero(self):
        policy = self.get_policy(origin="http://example.net", method='GET')
        self.assertTrue(policy.process_preflight_request())
        self.assertEqual('0', self.response.getHeader('Content-Length'))

    def test_preflight_cors_sets_status_code_200(self):
        policy = self.get_policy(origin="http://example.net", method='GET')
        self.assertTrue(policy.process_preflight_request())
        self.assertEqual(200, self.response.getStatus())


class TestCORS(unittest.TestCase):

    layer = PLONE_REST_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']

    def traverse(self, path='/plone', method='GET', headers={}):
        request = self.layer['request']
        request.environ['PATH_INFO'] = path
        request.environ['PATH_TRANSLATED'] = path
        request.environ['REQUEST_METHOD'] = method
        request.environ['HTTP_ORIGIN'] = 'http://example.net'
        request.environ.update(headers)
        notify(PubStart(request))
        return request.traverse(path)

    def test_preflight_cors_is_accessible_anonymously(self):
        headers = {}
        headers['HTTP_ACCESS_CONTROL_REQUEST_METHOD'] = 'POST'
        try:
            self.traverse(method='OPTIONS', headers=headers)
        except Unauthorized:
            self.fail('Service not accessible for preflight.')

    def test_simple_cors_gets_processed(self):
        headers = {}
        headers['HTTP_ACCEPT'] = 'application/json'
        obj = self.traverse(method='GET', headers=headers)
        obj()
        self.assertEqual(
            '*',
            self.request.response.getHeader('Access-Control-Allow-Origin'))

    def test_preflight_request_without_cors_policy_doesnt_render_service(self):
        # "Unregister" the current CORS policy
        class NoCORSPolicy(object):
            def __new__(cls, context, request):
                return None
        pushGlobalRegistry(self.portal)
        provideAdapter(
            NoCORSPolicy, (Interface, IDefaultBrowserLayer), ICORSPolicy)

        headers = {
            'HTTP_ACCESS_CONTROL_REQUEST_METHOD': 'GET',
            'HTTP_ACCEPT': '*/*',
        }
        service = self.traverse(method='OPTIONS', headers=headers)
        self.assertTrue(service() is None, 'Should return None')

        popGlobalRegistry(self.portal)



import os
from setuptools import setup, find_packages


def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

version = '1.3.0'

long_description = (
    read('README.rst') + '\n\n' +
    read('CHANGES.rst') + '\n\n'
    )


setup(name='plone.rest',
      version=version,
      description="Plone support for HTTP verbs.",
      long_description=long_description,
      # Get more strings from
      # http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
          "Development Status :: 5 - Production/Stable",
          "Environment :: Web Environment",
          "Framework :: Plone",
          "Framework :: Plone :: 4.3",
          "Framework :: Plone :: 5.0",
          "Framework :: Plone :: 5.1",
          "Framework :: Zope2",
          "License :: OSI Approved :: GNU General Public License (GPL)",
          "Operating System :: OS Independent",
          "Programming Language :: Python",
          "Programming Language :: Python :: 2.7",
      ],
      keywords='',
      author='Plone Foundation',
      author_email='plone-developers@lists.sourceforge.net',
      url='http://pypi.python.org/pypi/plone.rest',
      license='GPL version 2',
      packages=find_packages('src'),
      package_dir={'': 'src'},
      namespace_packages=['plone'],
      include_package_data=True,
      zip_safe=False,
      extras_require=dict(
          test=[
              'plone.app.testing[robot]>=4.2.2',
              'plone.app.robotframework',
              'plone.dexterity',
              'Products.CMFPlone',
              'requests',
          ]
      ),
      install_requires=[
          'setuptools',
          'collective.monkeypatcher',
          'zope.component',
          'zope.interface',
          'zope.publisher',
          'zope.traversing',
          'Products.CMFCore',
          'Zope2',
      ],
      entry_points="""
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



# -*- coding: utf-8 -*-
from plone.rest.cors import lookup_preflight_service_id
from plone.rest.interfaces import IAPIRequest
from plone.rest.negotiation import lookup_service_id
from zope.interface import alsoProvides


def mark_as_api_request(event):
    """Mark a request as IAPIRequest if there's a service registered for the
       actual request method and Accept header.
    """
    request = event.request
    method = request.get('REQUEST_METHOD', 'GET')
    if method == 'OPTIONS' and request.getHeader('Origin', False):
        preflighted_method = request.getHeader(
            'Access-Control-Request-Method', None)
        service_id = lookup_preflight_service_id(preflighted_method)
        request._rest_cors_preflight = True
    else:
        accept = request.getHeader('Accept', 'text/html')
        service_id = lookup_service_id(method, accept)
        request._rest_cors_preflight = False

    if service_id is not None:
        alsoProvides(request, IAPIRequest)
        request._rest_service_id = service_id

        # Flag as non-WebDAV request in order to avoid special treatment
        # in ZPublisher.BaseRequest.traverse().
        request.maybe_webdav_client = 0
